@@ -5,7 +5,7 @@
 ;; Author: Feng Shu <tumashu@163.com>
 ;; Maintainer: Feng Shu <tumashu@163.com>
 ;; URL: https://github.com/tumashu/posframe
-;; Version: 1.4.2
+;; Version: 1.5.1
 ;; Keywords: convenience, tooltip
 ;; Package-Requires: ((emacs "26.1"))
 
@@ -58,6 +58,15 @@ case, but suggest use function `posframe-mouse-banish-simple' or
 custom function for EXWM users."
   :type 'function)
 
+(defcustom posframe-text-scale-factor-function #'posframe-text-scale-factor-default
+  "The function to adjust value of text-scale of posframe buffer.
+
+Accepts single argument which is the value of parent buffer
+`text-scale-mode-amount' or nil if the `text-scale-mode' is disabled in
+the parent buffer."
+  :group 'posframe
+  :type 'function)
+
 (defvar-local posframe--frame nil
   "Record posframe's frame.")
 
@@ -123,9 +132,11 @@ effect.")
 (defun posframe-workable-p ()
   "Test posframe workable status."
   (and (>= emacs-major-version 26)
-       (not (or noninteractive
-                emacs-basic-display
-                (not (display-graphic-p))))))
+       (not noninteractive)
+       (not emacs-basic-display)
+       (or (display-graphic-p)
+           (featurep 'tty-child-frames))
+       (eq (frame-parameter (selected-frame) 'minibuffer) 't)))
 
 ;;;###autoload
 (cl-defun posframe-show (buffer-or-name
@@ -150,6 +161,7 @@ effect.")
                          internal-border-color
                          font
                          cursor
+                         tty-non-selected-cursor
                          window-point
                          foreground-color
                          background-color
@@ -206,7 +218,8 @@ position.  Its argument is a plist of the following form:
    :header-line-height xxx
    :tab-line-height xxx
    :x-pixel-offset xxx
-   :y-pixel-offset xxx)
+   :y-pixel-offset xxx
+   :parent-text-scale-mode-amount xxx)
 
 By default, poshandler is auto-selected based on the type of POSITION,
 but the selection can be overridden using the POSHANDLER argument.
@@ -217,21 +230,22 @@ The builtin poshandler functions are listed below:
 2.  `posframe-poshandler-frame-top-center'
 3.  `posframe-poshandler-frame-top-left-corner'
 4.  `posframe-poshandler-frame-top-right-corner'
-5.  `posframe-poshandler-frame-bottom-center'
-6.  `posframe-poshandler-frame-bottom-left-corner'
-7.  `posframe-poshandler-frame-bottom-right-corner'
-8.  `posframe-poshandler-window-center'
-9.  `posframe-poshandler-window-top-center'
-10. `posframe-poshandler-window-top-left-corner'
-11. `posframe-poshandler-window-top-right-corner'
-12. `posframe-poshandler-window-bottom-center'
-13. `posframe-poshandler-window-bottom-left-corner'
-14. `posframe-poshandler-window-bottom-right-corner'
-15. `posframe-poshandler-point-top-left-corner'
-16. `posframe-poshandler-point-bottom-left-corner'
-17. `posframe-poshandler-point-bottom-left-corner-upward'
-18. `posframe-poshandler-point-window-center'
-19. `posframe-poshandler-point-frame-center'
+5.  `posframe-poshandler-frame-top-left-or-right-other-corner'
+6.  `posframe-poshandler-frame-bottom-center'
+7.  `posframe-poshandler-frame-bottom-left-corner'
+8.  `posframe-poshandler-frame-bottom-right-corner'
+9.  `posframe-poshandler-window-center'
+10. `posframe-poshandler-window-top-center'
+11. `posframe-poshandler-window-top-left-corner'
+12. `posframe-poshandler-window-top-right-corner'
+13. `posframe-poshandler-window-bottom-center'
+14. `posframe-poshandler-window-bottom-left-corner'
+15. `posframe-poshandler-window-bottom-right-corner'
+16. `posframe-poshandler-point-top-left-corner'
+17. `posframe-poshandler-point-bottom-left-corner'
+18. `posframe-poshandler-point-bottom-left-corner-upward'
+19. `posframe-poshandler-point-window-center'
+20. `posframe-poshandler-point-frame-center'
 
  (3) POSHANDLER-EXTRA-INFO
 
@@ -283,11 +297,15 @@ derived from the current frame by default, but can be overridden
 using the FONT, FOREGROUND-COLOR and BACKGROUND-COLOR arguments,
 respectively.
 
- (10) CURSOR and WINDOW-POINT
+ (10) CURSOR, TTY-NON-SELECTED-CURSOR and WINDOW-POINT
 
 By default, cursor is not showed in posframe, user can let cursor
 showed with this argument help by set its value to a `cursor-type'.
 
+TTY-NON-SELECTED-CURSOR will let redisplay put the terminal
+cursor in a non-selected frame, which is useful when use
+vertico-posframe like package in tty.
+
 When cursor need to be showed in posframe, user may need to set
 WINDOW-POINT to the point of BUFFER, which can let cursor showed
 at this point.
@@ -355,7 +373,7 @@ when it is nil or it return nil, child-frame feature will be used
 and reference position will be deal with in Emacs.
 
 The user case I know at the moment is let ivy-posframe work well
-in EXWM environment (let posframe show on the other appliction
+in EXWM environment (let posframe show on the other application
 window).
 
          DO NOT USE UNLESS NECESSARY!!!
@@ -402,7 +420,11 @@ You can use `posframe-delete-all' to delete all posframes."
          (font-width (default-font-width))
          (font-height (with-current-buffer (window-buffer parent-window)
                         (posframe--get-font-height position)))
-         (mode-line-height (window-mode-line-height))
+         (parent-text-scale-mode-amount (with-current-buffer (window-buffer parent-window)
+                                          (and (bound-and-true-p text-scale-mode) text-scale-mode-amount)))
+         (mode-line-height (window-mode-line-height
+                            (and (window-minibuffer-p)
+                                 (ignore-errors (window-in-direction 'above)))))
          (minibuffer-height (window-pixel-height (minibuffer-window)))
          (header-line-height (window-header-line-height parent-window))
          (tab-line-height (if (functionp 'window-tab-line-height)
@@ -428,6 +450,7 @@ You can use `posframe-delete-all' to delete all posframes."
              :position position
              :font font
              :cursor cursor
+             :tty-non-selected-cursor tty-non-selected-cursor
              :parent-frame
              (unless ref-position
                parent-frame)
@@ -444,7 +467,8 @@ You can use `posframe-delete-all' to delete all posframes."
              :respect-header-line respect-header-line
              :respect-mode-line respect-mode-line
              :override-parameters override-parameters
-             :accept-focus accept-focus))
+             :accept-focus accept-focus
+             :parent-text-scale-mode-amount parent-text-scale-mode-amount))
 
       ;; Insert string into the posframe buffer
       (posframe--insert-string string no-properties)
@@ -472,8 +496,14 @@ You can use `posframe-delete-all' to delete all posframes."
                        :font-height font-height
                        :font-width font-width
                        :posframe posframe
-                       :posframe-width (frame-pixel-width posframe)
-                       :posframe-height (frame-pixel-height posframe)
+                       :posframe-width
+                       (if width
+                           (* (default-font-width) width)
+                         (frame-pixel-width posframe))
+                       :posframe-height
+                       (if height
+                           (* (default-line-height) height)
+                         (frame-pixel-height posframe))
                        :posframe-buffer buffer
                        :parent-frame parent-frame
                        :parent-frame-width parent-frame-width
@@ -493,7 +523,8 @@ You can use `posframe-delete-all' to delete all posframes."
                        :header-line-height header-line-height
                        :tab-line-height tab-line-height
                        :x-pixel-offset x-pixel-offset
-                       :y-pixel-offset y-pixel-offset))))
+                       :y-pixel-offset y-pixel-offset
+                       :parent-text-scale-mode-amount parent-text-scale-mode-amount))))
 
       ;; Move posframe
       (posframe--set-frame-position
@@ -573,12 +604,14 @@ You can use `posframe-delete-all' to delete all posframes."
                                      internal-border-color
                                      font
                                      cursor
+                                     tty-non-selected-cursor
                                      keep-ratio
                                      lines-truncate
                                      override-parameters
                                      respect-header-line
                                      respect-mode-line
-                                     accept-focus)
+                                     accept-focus
+                                     parent-text-scale-mode-amount)
   "Create and return a posframe child frame.
 This posframe's buffer is BUFFER-OR-NAME.
 
@@ -598,6 +631,7 @@ ACCEPT-FOCUS."
         (after-make-frame-functions nil)
         (x-gtk-resize-child-frames posframe-gtk-resize-child-frames)
         (args (list "args"
+                    (display-graphic-p)
                     foreground-color
                     background-color
                     right-fringe
@@ -689,10 +723,14 @@ ACCEPT-FOCUS."
                        (line-spacing . 0)
                        (unsplittable . t)
                        (no-other-frame . t)
-                       (undecorated . t)
+                       ;; NOTE: TTY child frame use undecorated to control border.
+                       (undecorated . ,(or (display-graphic-p)
+                                           (not (and (> border-width 0)
+                                                     (featurep 'tty-child-frames)))))
                        (visibility . nil)
-                       (cursor-type . nil)
-                       (minibuffer . nil)
+                       (cursor-type . ,cursor)
+                       (tty-non-selected-cursor . ,tty-non-selected-cursor)
+                       (minibuffer . ,(minibuffer-window parent-frame))
                        (left . ,(if (consp position) (car position) 0))
                        (top . ,(if (consp position) (cdr position) 0))
                        (width . 1)
@@ -708,7 +746,7 @@ ACCEPT-FOCUS."
          (or font (face-attribute 'default :font parent-frame)))
         (when border-color
           (if parent-frame
-	      (set-face-background
+              (set-face-background
                (if (facep 'child-frame-border)
                    'child-frame-border
                  'internal-border)
@@ -749,6 +787,10 @@ ACCEPT-FOCUS."
       ;; so we should force set parent-frame again in this place.
       (set-frame-parameter posframe--frame 'parent-frame parent-frame)
 
+      ;; Set text scale based on the parent frame text scale.
+      (text-scale-set
+       (funcall posframe-text-scale-factor-function parent-text-scale-mode-amount))
+
       posframe--frame)))
 
 (defun posframe--find-existing-posframe (buffer &optional last-args)
@@ -811,8 +853,14 @@ will be removed."
         (max-height (plist-get size-info :max-height))
         (min-width (plist-get size-info :min-width))
         (min-height (plist-get size-info :min-height)))
-    (when height (set-frame-height posframe height))
-    (when width (set-frame-width posframe width))
+    (cond
+     ((and width height)
+      (set-frame-size posframe
+                      (* (default-font-width) width)
+                      (* (default-line-height) height)
+                      t))
+     (height (set-frame-height posframe height))
+     (width (set-frame-width posframe width)))
     (unless (and height width)
       (posframe--fit-frame-to-buffer
        posframe max-height min-height max-width min-width
@@ -884,7 +932,7 @@ of `posframe-show'."
               (t (error "Posframe: have no valid poshandler"))))))
 
 (defun posframe--calculate-new-position (info position ref-position)
-  "Calcuate new position according to INFO, POSITION and REF-POSITION."
+  "Calculate new position according to INFO, POSITION and REF-POSITION."
   (let* ((parent-frame-width (plist-get info :parent-frame-width))
          (parent-frame-height (plist-get info :parent-frame-height))
          (posframe-width (plist-get info :posframe-width))
@@ -913,7 +961,8 @@ This need PARENT-FRAME-WIDTH and PARENT-FRAME-HEIGHT"
                (equal posframe--last-posframe-displayed-size
                       (cons (frame-pixel-width posframe)
                             (frame-pixel-height posframe))))
-    (set-frame-position posframe (car position) (cdr position))
+    (unless (equal (frame-position posframe) position)
+      (set-frame-position posframe (car position) (cdr position)))
     (setq-local posframe--last-posframe-pixel-position position)
     (setq-local posframe--last-parent-frame-size
                 (cons parent-frame-width parent-frame-height))
@@ -1281,6 +1330,25 @@ The structure of INFO can be found in docstring of
 `posframe-show'."
   '(-1 . 0))
 
+(defun posframe-poshandler-frame-top-left-or-right-other-corner (info)
+  "Posframe's position handler.
+
+This poshandler function let posframe align to top left or top right
+corner of frame, based on whether current window is relatively at left
+or right in the current frame.  If window is at left, place posframe on
+right, and vice versa.  (This is calculated by whether current window
+center is left or right to frame center.)
+
+The structure of INFO can be found in docstring of `posframe-show'."
+  (let ((window-left (plist-get info :parent-window-left))
+        (window-width (plist-get info :parent-window-width))
+        (frame-width (plist-get info :parent-frame-width)))
+    ;; when equal, put posframe on right because content in window tend to be on left
+    (if (<= (+ window-left (/ window-width 2))
+            (/ frame-width 2))
+        '(-1 . 0)
+      '(0 . 0))))
+
 (defun posframe-poshandler-frame-bottom-left-corner (info)
   "Posframe's position handler.
 
@@ -1475,6 +1543,11 @@ window manager selects it."
              (not posframe--accept-focus))
     (redirect-frame-focus posframe--frame (frame-parent))))
 
+(defun posframe-text-scale-factor-default (parent-text-scale-mode-amount)
+  "Return PARENT-TEXT-SCALE-MODE-AMOUNT or 0 if it is nil.
+This ensures text scale factor is always a number for posframe display."
+  (or parent-text-scale-mode-amount 0))
+
 (provide 'posframe)
 
 ;;; posframe.el ends here
